import ij.*;
import ij.process.*;
import ij.gui.*;
import java.awt.*;
import ij.plugin.*;
import ij.plugin.frame.*;
import ij.plugin.filter.PlugInFilter;
import ij.*;
import ij.io.*;
import ij.plugin.*;
import ij.plugin.filter.*;

import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.util.zip.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.lang.InterruptedException;
import java.lang.System;
import java.lang.Math.*;

///Plugin class to load up a QuickPalm run
///
///@ingroup gPlugin
public class  CoxGroupQuickPalmLoader implements PlugIn {
	static 
	{
		new CoxGroupCheckForUpdates().run("boot");
	}

	private void close(InputStream i)
	{
		//Not sure what to do here...
		if(i != null)
		{
			try{
				i.close();
			}
			catch(IOException ioe)
			{
				Toolkit.getDefaultToolkit().beep();
				ij.IJ.showStatus("Error closing file: " + ioe.getMessage());
			}
		}
	}

	public void run(String arg) {
		InputStream in = null;
		String name;
		double ini_FWHM=100., ini_reconstruction_pixel_size=30., ini_sigma_filter = 0;
		boolean show_control_panel=true;
		boolean do_filtering=true;

		try{

			OpenDialog o = new OpenDialog("Open QuickPalm data...", null);
			name = o.getFileName();
			
			try{
				in = new FileInputStream(o.getDirectory() + o.getFileName());
			}
			catch(java.io.FileNotFoundException ferr)
			{
				Toolkit.getDefaultToolkit().beep();
				ij.IJ.showStatus("Error opening file: " + ferr.getMessage());
				return;
			}



			GenericDialog g = new GenericDialog("Reconstruction Parameters");
			g.addNumericField("FWHM (initial) ", ini_FWHM, 0, 5, "nm");
			g.addNumericField("Reconstruction size (initial)", ini_reconstruction_pixel_size, 0, 5, "nm");
			g.addNumericField("Small localisation filter (initial)", ini_sigma_filter, 0, 5, "nm");

			g.addCheckbox("Show control panel", show_control_panel);
			
			((Checkbox)g.getCheckboxes().get(0)).hide();
			
			g.showDialog();

			ini_FWHM = g.getNextNumber();
			ini_reconstruction_pixel_size=g.getNextNumber();
			show_control_panel=g.getNextBoolean();
			ini_sigma_filter = g.getNextNumber();
		

			//Yay @ cargoculting
			InputStreamReader d = new InputStreamReader(in);
			BufferedReader r = new BufferedReader(d);

			String line;
			final ArrayList<Spot> spots = new ArrayList<Spot>();
			int iterations=0;
			Rectangle roi=null;


			int line_number=0;
			int frames=0;
			double max_val = 0; //Assume square image: almost all scientific imagers are close enough to square
			double pixel_size_in_nm = -1;
			for(  ; (line = r.readLine()) != null; line_number++)
			{
				//discard the header

				if(line_number == 0)
					continue;
				String tokens[] = line.split("\\p{Space}+");
				
				//Tokens are:
				//number intensity xpx ypx xnm ynm ...

				if((tokens.length) < 15)
					throw new IOException("corrupt file (short line)");

				Spot s = new Spot();
				try
				{
					//QuickPalm outputs both pixels and nm. We could use the nm and fake the
					//pixel size except that the widths are in pixels, not nm.
					//Average it because the output file is quite heavily quantized.
					pixel_size_in_nm += Double.parseDouble(tokens[5]) / Double.parseDouble(tokens[3]);
					pixel_size_in_nm += Double.parseDouble(tokens[4]) / Double.parseDouble(tokens[2]);


					s.x = Double.parseDouble(tokens[2]);
					s.y = Double.parseDouble(tokens[3]);
					
					double wl = Double.parseDouble(tokens[7]);
					double wh = Double.parseDouble(tokens[8]);
					double hl = Double.parseDouble(tokens[9]);
					double hh = Double.parseDouble(tokens[10]);
					s.sigma = Math.min(wl+wh, hl+hh);


					//QuickPalm has the frame number as a double, but is always integral
					int frame = (int)Double.parseDouble(tokens[14]);

					if(frame > frames)
						frames = frame;

					if(s.x > max_val)
						max_val = s.x;
					if(s.y > max_val)
						max_val = s.y;
				}
				catch(NumberFormatException nerr)
				{
					throw new IOException("corrupt file (bad spot position)");
				}

				spots.add(s);
			}

			roi = new Rectangle(0, 0, (int)Math.ceil(max_val), (int)Math.ceil(max_val));
			
			pixel_size_in_nm /= (2*(line_number-1));
			System.out.println("pixel_size_in_nm = " + pixel_size_in_nm);

			if(show_control_panel)
			{
				final String fname = name;
				final Rectangle roi_ = roi;
				final int its = iterations;
				final double ini_FWHM_ = ini_FWHM;
				final double ini_reconstruction_pixel_size_ = ini_reconstruction_pixel_size;
				final String msg = "Using " + fname + ": " + Integer.toString(frames) + " frames, " + Integer.toString(spots.size()) + " localisations .";
				final double pixel_size_in_nm_ = pixel_size_in_nm;
				final double ini_sigma_filter_ = ini_sigma_filter;
				SwingUtilities.invokeLater(
				new Runnable() {
						public void run() {
								CoxGroupReconstructionViewer e = new CoxGroupReconstructionViewer(roi_, pixel_size_in_nm_, fname, null, true);
								e.append(spots, 100000); // 100000 == frames, to stop "Export" button from being red as a warning. This qualifies as lolhax
								e.send_update_canvas_event();
								e.send_status_text_message(msg);
								e.set_reconstructed_pixel_size(ini_reconstruction_pixel_size_);
								e.set_reconstruction_blur_fwhm(ini_FWHM_);
								e.set_sigma_filter(ini_sigma_filter_);
								e.send_update_canvas_event();
						}

				}
				);
			}
			else
			{
				System.out.println("etf\n");
				double zoom=pixel_size_in_nm / ini_reconstruction_pixel_size; 
				ImagePlus   linear_reconstruction;
				linear_reconstruction = new ImagePlus();
				linear_reconstruction.setProcessor(Reconstruction.reconstruct(roi, zoom, ini_FWHM, pixel_size_in_nm, spots, ini_sigma_filter));
				linear_reconstruction.show();
			}




		}
		catch(IOException another_ioe){
			Toolkit.getDefaultToolkit().beep();
			ij.IJ.showStatus("Error reading data: " + another_ioe.getMessage());
		}
		finally{
			close(in);
		}

	}



}
