

if ! git status | awk '$1=="modified:"{exit 1}'
then
	echo "[31mThere are uncommitted changes. This means that the build" 1>&2
	echo "Will not represent a sane version.[0m" 1>&2
	extra="plus uncommitted changes"
	time=`date +%s`
else
	time=`git log -1 --format=%at HEAD`
	extra=
fi

git_hash=`git rev-parse HEAD`
tags=

if git describe --tags > /dev/null
then
	tags="$(git tag --points-at HEAD)"

	if echo "$tags" | grep -q VERSION
	then 
		version="$(echo "$tags" | awk -F_ '/VERSION/{print $2}')"
	fi
fi

date="`date --date @$time +'%a %b %d, %Y'`"


if test "$1" = "format"
then
	if test "$extra" = ""
	then 
		if test "$version" != ""
		then
			echo "Version: $version"
		fi

		echo "Date: $date<br>"
		echo "Tags: $tags<br>"
		echo "Branch: $(git branch | awk '{print $2}') <br>"
		echo "Git hash: $git_hash $extra <br>"
		echo "Version code: $time <br>"
	else
		echo "Git hash: $git_hash $extra <br>"
		echo "Date: $date<br>"
		echo "Branch:  $(git branch | awk '{print $2}') <br>"
	fi
else
	echo $time
fi
